require 'google_chart'

DEFAULT_COLORS = [ 'ff0000', '00ff00', '0000ff', 'ffff00', '00ffff', 'ff00ff' ]

DAY_SEC = 60 * 60 * 24

DAY_GRANULARITY_FUNC = proc do |t|
  ((t - Time.mktime(1980, 1, 1)) / DAY_SEC).to_i
end

NO_COMPRESS = proc{|cnt|cnt}

WEEK_COMPRESS = proc do |cnt|
  i = 0
  r = Array.new(cnt.size/7+1){0}
  while i < cnt.size
    r[i/7] += cnt[i]
    i+=1
  end
  r
end

WEEK_STRS = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat']

def times_to_hist(times,
                  granularity_func = DAY_GRANULARITY_FUNC)
  days = Hash.new{0}
  times.each do |t|
    days[granularity_func[t]] += 1
  end
  min = days.keys.min
  max = days.keys.max

  cnts = []
  min.upto(max) do |d|
    cnts << days[d]
  end

  cnts
end

def time_hist_gv(hist, title, start_time, annotates = {})
  t = start_time
  a = []
  hist.each do |c|
    key = [t.year, t.mon, t.day]
    an = annotates[key]
    key << c
    key << "'#{an}'" if an
    a << "[#{key * ','}]"
    t += DAY_SEC
  end

  s = %Q(
<script type="text/javascript" src="http://www.google.com/jsapi"></script>
<script type="text/javascript">
  google.load("visualization", "1", {packages:["annotatedtimeline"]});
  google.setOnLoadCallback(drawChart);
  function drawChart() {
    var data = new google.visualization.DataTable();
    data.addColumn('date', 'Date');
    data.addColumn('number', '#{title}');
    data.addColumn('string', 'title1');
    //data.addColumn('string', 'text1');
    var DATA = [
#{a * ",\n"}
    ];
    data.addRows(DATA.length);
    var lastDate;
    for (var i = 0; i < DATA.length; i++) {
      var d = DATA[i];
      var date = new Date(d[0], d[1]-1, d[2]);
      lastDate = date;
      data.setValue(i, 0, date);
      data.setValue(i, 1, d[3]);
      if (d[4]) data.setValue(i, 2, d[4]);
    }

    var zoomStartDate = new Date();
    zoomStartDate.setTime(lastDate.getTime() - 180 * 24 * 60 * 60 * 1000);

    var chartDiv = document.getElementById('chart_div');
    var chart = new google.visualization.AnnotatedTimeLine(chartDiv);
    chart.draw(data, {displayAnnotations: true,
                      allowHtml: true,
                      zoomStartTime: zoomStartDate,
                     });
  }
</script>
)

end

def time_hist_chart(kinds, hist, title, start_time,
                    display_compress_func = NO_COMPRESS,
                    colors = DEFAULT_COLORS)
  s = ''
  GoogleChart::LineChart.new('600x400', title, false) do |chart|
    s << '<table><tr><td>'
    xmin = 0
    xmax = 0
    ymax = 0
    sizemax = 0
    kinds.sort.zip(colors).each do |k, color|
      cnt = hist[k]
      cnt = display_compress_func[cnt]
      ymax = [ymax, cnt.max].max
      sizemax = [sizemax, cnt.size].max
      chart.data k, cnt, color
    end
    chart.axis :y, :range => [0, ymax]
    first = start_time.strftime('%Y%m%d')
    last = (start_time + (sizemax-1) * DAY_SEC).strftime('%Y%m%d')
    chart.axis :x, :labels => [first, last]

    s << %Q(<img src="#{chart.to_url}">)

    s << '</td><td>'

    s << '<table border="1">'
    s << '<tr><th></th><th>Total</th><th>Avg</th><th>Max</th><th>Min</th></tr>'
    num_data = display_compress_func[hist['all']].size
    kinds.sort.each do |k|
      total = 0
      max = nil
      min = nil
      display_compress_func[hist[k]].each do |v|
        max = v if !max || v > max
        min = v if !min || v < min
        total += v
      end
      avg = total / num_data
      s << '<tr>'
      [k, total, avg, max, min].each do |v|
        s << "<td>#{v}</td>"
      end
      s << '</tr>'
    end
    s << '</table>'
    s << '</td></table>'
  end
  s
end

def topn_pie_chart(hist, title, topnum)
  s = ''
  GoogleChart::PieChart.new('600x400', title, false) do |chart|
    s << '<table><tr><td>'

    sorted = hist.sort_by{|k,v|-v}
    other = 0
    30.upto(sorted.size-1) do |i|
      other += sorted[i][1]
    end
    total = other
    top = sorted[0, topnum]
    top.each do |k, v|
      total += v
      chart.data k, v
    end
    chart.data 'other', other if other > 0
    s << %Q(<img src="#{chart.to_url}">)

    s << '</td><td>'
    s << '<table border="1">'
    i = 0
    top.each do |k, v|
      s << '<tr>'
      s << "<th>#{i+=1}</th><th>#{k}</th>"
      s << "<td>#{v}</td><td>#{'%.2f'%(v.to_f*100/total)}%</td>"
      s << '</tr>'
    end
    s << '</table>'
    s << '</td></table>'
  end
  s
end

def count_by_hour_chart_impl(d, title,
                             colors = DEFAULT_COLORS)
  s = ''
  GoogleChart::BarChart.new('600x400', title, :vertical, false) do |chart|
    chart.data 'all', d, colors[1]
    chart.axis(:x, :labels => [*0..23])
    chart.axis(:y, :range => [0, d.max])
    chart.width_spacing_options :bar_width => 14
    s << %Q(<img src="#{chart.to_url}">)
  end
  s
end

def count_by_hour_chart(times, title,
                        colors = DEFAULT_COLORS)
  d = Array.new(24){0}
  times.each do |t|
    d[t.hour] += 1
  end

  count_by_hour_chart_impl(d, title, colors)
end

def count_by_wday_chart_impl(wdata, title,
                             colors = DEFAULT_COLORS)
  s = ''
  GoogleChart::BarChart.new('600x260', title, :horizontal, false) do |chart|
    chart.data 'all', wdata, colors[2]
    chart.axis(:x, :range => [0, wdata.max])
    chart.axis(:y, :labels => WEEK_STRS.reverse)
    #chart.width_spacing_options :group_spacing => 30
    s << %Q(<img src="#{chart.to_url}">)
  end
  s
end

def count_by_wday_chart(times, title,
                        colors = DEFAULT_COLORS)
  wdata = [0] * 7
  times.each do |t|
    wdata[t.wday] += 1
  end

  count_by_wday_chart_impl(wdata, title, colors)
end

