#!/usr/bin/env ruby

require 'chartutil'

TOP_CMDS_FOR_PIE_CHART = 20

if !ARGV[0]
  puts "Usage: #$0 zhistory [user]"
  exit 1
end

user = ARGV[1] || ENV['USER']

cmds = Hash.new{0}
times = []

File.readlines(ARGV[0]).each do |line|
  if line !~ /^: (\d+):(\d+);(\S*)/
    next
  end

  time = $1
  elapsed = $2
  cmd = $3

  cmds[cmd] += 1
  times << Time.at(time.to_i)
end

start_time = times[0]
end_time = times[-1]

annotate_url = nil
annotates = {}
if ARGV[2]
  File.open(ARGV[2]) do |ifile|
    annotate_url = ifile.gets.chomp
    if annotate_url =~ /^$/
      annotate_url = nil
    end

    while l=ifile.gets.chomp
      if l !~ /((\d\d\d\d),(\d\d),(\d\d)),(.*)/
        raise "Unknown format: #{ARGV[2]}"
      end
      t = Time.mktime(y=$2.to_i, m=$3.to_i, d=$4.to_i)
      next if t > end_time
      break if t < start_time

      b = $5.gsub("\'",'\&\&')
      if annotate_url
        url = t.strftime(annotate_url)
        b = %Q(<a href="#{url}">#{b}</a>)
      end
      annotates[[y,m,d]] = b
    end
  end
end

hist = {'all' => times_to_hist(times)}

TITLE = "#{user}\'s zhistory statistics"

gv = time_hist_gv(hist['all'], 'Commands / day', start_time, annotates)

puts %Q(<html>
<head>
<title>#{TITLE}</title>
#{gv}
</head>
<body>
<h1>#{TITLE}</h1>
<div id="chart_div" style="width: 900px; height: 400px;"></div>
)

title = '# Commands / day'
puts time_hist_chart(['all'], hist, title, start_time)

title = '# Commands / week'
puts time_hist_chart(['all'], hist, title, start_time, WEEK_COMPRESS)

title = "Top #{TOP_CMDS_FOR_PIE_CHART} commands"
puts topn_pie_chart(cmds, title, TOP_CMDS_FOR_PIE_CHART)

title = '# Commands by hour'
puts '<p>'
puts count_by_hour_chart(times, title)

title = '# Commands by wday'
puts '<p>'
puts count_by_wday_chart(times, title)

puts %Q(</body></html>)
